local function listFormat(list)
	local tmpList = {}
	for _,v in pairs(list) do
		table.insert(tmpList, v)
	end
	table.sort(tmpList)
	tmpString = ''
	for k,v in pairs(list) do
		local item_link = select(2, GetItemInfo(k))
		if item_link == nil then
			item_link = v
		end
		if #tmpString > 0 and #tmpString + #item_link <= 255 then
			tmpString = tmpString .. ', ' .. item_link
		else
			if #tmpString == 0 then
				tmpString = item_link
			else
				print(tmpString)
				tmpString = item_link
			end
		end
	end
	if #tmpString ~= 0 then
		print(tmpString)
	end
end

local function listRemove(list, item)
	found = false
	for k,v in pairs(list) do
		if string.lower(v) == string.lower(item) then
			list[k] = nil
			found = true
		end
	end
	return found
end

local function listToggle(list, listName, itemId, itemName)
	if list[itemId] then
		list[itemId] = nil
		print('Removed '..itemName..' from '..listName..'.')
	else
		table.insert(list, itemId, itemName)
		print('Added '..itemName..' to '..listName..'.')
	end
end

local function initVars()
	if AutoVendor == nil or type(AutoVendor) ~= "table" then
		AutoVendor = {}
	end
	if AutoVendor['JunkList'] == nil or type(AutoVendor['JunkList']) ~= "table" then
		AutoVendor['JunkList'] = {}
	end
	if AutoVendor['NotJunkList'] == nil or type(AutoVendor['NotJunkList']) ~= "table" then
		AutoVendor['NotJunkList'] = {}
	end
	if not AutoVendor['SavedVariablesVersion'] then
		AutoVendor['SavedVariablesVersion'] = -1
	end
	if AutoVendor['SavedVariablesVersion'] < AUTOVENDOR_CURRENT_VERSION then
		for k,v in pairs(AUTOVENDOR_NOT_JUNK) do
			if k > AutoVendor['SavedVariablesVersion'] then
				for k2,v2 in pairs(v) do
					AutoVendor['NotJunkList'][k2] = v2
				end
			end
		end
	end
	
	if not AutoVendor['Verbosity'] then 
		AutoVendor['Verbosity'] = AUTOVENDOR_VERBOSITY_SUMMARY 
	end
	
	AutoVendor['SavedVariablesVersion'] = AUTOVENDOR_CURRENT_VERSION
end

function SlashCmdList.AUTOVENDORTOGGLEJUNK(msg, editbox)
	if msg then
		initVars()
		local itemId = tonumber(strmatch(msg, "item:(%d+)"))
		local itemName = select(1, GetItemInfo(msg))
		if itemId and itemName then
			listToggle(AutoVendor['JunkList'], 'junk list', itemId, itemName)
		else
			if msg and listRemove(AutoVendor['JunkList'], msg) then
				print('Removed '..msg..' from junk list.')
			else
				print('No item (link) supplied!')
			end
		end
	else
		print('No item link supplied!')
	end
end
function SlashCmdList.AUTOVENDORTOGGLENOTJUNK(msg, editbox)
	if msg then
		initVars()
		local itemId = tonumber(strmatch(msg, "item:(%d+)"))
		local itemName = select(1, GetItemInfo(msg))
		if itemId and itemName then
			listToggle(AutoVendor['NotJunkList'], 'not junk list', itemId, itemName)
		else
			if msg and listRemove(AutoVendor['NotJunkList'], msg) then
				print('Removed '..msg..' from not junk list.')
			else
				print('No item (link) supplied!')
			end
		end
	else
		print('No item (link) supplied!')
	end
end

function SlashCmdList.AUTOVENDORJUNKLIST(msg, editbox)
	initVars()
	print('Items in the junk list:')
	listFormat(AutoVendor['JunkList'])
end

function SlashCmdList.AUTOVENDORNOTJUNKLIST(msg, editbox)
	initVars()
	print('Items in the not junk list:')
	listFormat(AutoVendor['NotJunkList'])
end

function SlashCmdList.AUTOVENDORVERBOSITY(msg, editbox)
	if msg and msg == '0' or msg == '1' or msg == '2' then
		if msg == '0' then
			print('Turning AutoVendor reports off.')
		end
		if msg == '1' then
			print('AutoVendor will report a summary.')
		end
		if msg == '2' then
			print('AutoVendor will report all of its sales.')
		end
		AutoVendor['Verbosity'] = tonumber(msg)
	else
		print('Usage: /junkreport 0-2')
		print('  Where 0 means no reports, 1 means summary reports and 2 means full reports.')
	end
end

local function OnEvent()
	initVars()
	local iconSize = select(2, GetChatWindowInfo(1)) + 6
	local totalSellValue = 0
	local totalItemsSold = 0
	for bag=0,4 do
		for slot=0,GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if link then
				local itemId = tonumber(strmatch(link, "item:(%d+)"))
				if (select(3, GetItemInfo(link)) == 0 or AutoVendor['JunkList'][itemId]) and not AutoVendor['NotJunkList'][itemId] and itemId ~= 6196 then
					ShowMerchantSellCursor(1)
					UseContainerItem(bag, slot)
					local itemCount = select(2, GetContainerItemInfo(bag, slot))
					local sellValue = itemCount * select(11, GetItemInfo(link))
					totalSellValue = totalSellValue + sellValue
					totalItemsSold = totalItemsSold + 1
					if AutoVendor['Verbosity'] >= AUTOVENDOR_VERBOSITY_ALL then
						print(format("Selling %sx%d for %s.", link, itemCount, GetCoinTextureString(sellValue, iconSize)))
					end
				end
			end
		end
	end
	if totalItemsSold > 0 and AutoVendor['Verbosity'] >= AUTOVENDOR_VERBOSITY_SUMMARY then
		local items = 'items'
		if totalItemsSold == 1 then
			items = 'item'
		end
		print(format("Automatically sold %d %s for %s.", totalItemsSold, items, GetCoinTextureString(totalSellValue, iconSize)))
	end
end




AUTOVENDOR_CURRENT_VERSION = 0.3
AUTOVENDOR_VERBOSITY_NONE = 0
AUTOVENDOR_VERBOSITY_SUMMARY = 1
AUTOVENDOR_VERBOSITY_ALL = 2
AUTOVENDOR_NOT_JUNK = {
	[0.1] = {
		[1485] = "Pitchfork",
		[39202] = "Rusted Pitchfork",
		[3944] = "Twill Belt",
		[3945] = "Twill Boots",
		[3946] = "Twill Bracers",
		[3947] = "Twill Cloak",
		[8754] = "Twill Cover",
		[3948] = "Twill Gloves",
		[3949] = "Twill Pants",
		[3950] = "Twill Shoulderpads",
		[3951] = "Twill Vest",
	},
	[0.4] = {
		[18230] = "Broken I.W.I.N. Button",
	}
}




SLASH_AUTOVENDORTOGGLEJUNK1 = '/junk'
SLASH_AUTOVENDORTOGGLENOTJUNK1 = '/notjunk'
SLASH_AUTOVENDORJUNKLIST1 = '/junklist'
SLASH_AUTOVENDORNOTJUNKLIST1 = '/notjunklist'
SLASH_AUTOVENDORVERBOSITY1 = '/junkreport'

local f = CreateFrame("Frame")
f:RegisterEvent("MERCHANT_SHOW")
f:SetScript("OnEvent", OnEvent)

if MerchantFrame:IsVisible() then OnEvent() end
